#ifndef AGENT_H
#define AGENT_H

#include <QThread>
#include <QMap>
#include <QDataStream>
#include <QQueue>
#include <QString>
#include <QMutex>
#include <QMutexLocker>
#include <QUuid>
#include <QIODevice>
#include <QTextStream>

#include "Exception.h"

#include "RFIDData.h"
#include "VictimRFIDData.h"

#include "XSocket.h"

class Sensor;

class Agent : public QThread {

private:
	static const int TIMEOUT;
	
	bool logWrite;
	QIODevice* logDevice;
	QTextStream* logStream;

public:

	static const uint RENDERINGMODE_NONE   = 0;
	static const uint RENDERINGMODE_SIMPLE = 1;
	static const uint RENDERINGMODE_SLEEK  = 2;

	
	Agent();
	virtual ~Agent();

    virtual void configure(QString model, QString name, QString position, QString rotation);

    QUuid   getUniqueID();
    QString getUniqueName();
    QString getName();
    
    void logTo(QIODevice*);
    void logFrom(QIODevice*);

	void connect(const QString &host, const quint16 port); 
	void disconnect();
	
	void run();
	
	void send(QString cmd);

    virtual void notifyUpdate(Sensor* sensor) {};

	virtual void setRenderingMode(uint mode);

    virtual void saveMap(int pxPerMeter, QString filename) {}
    virtual void saveRFIDs(QString filename) {}
    virtual void saveVictims(QString filename) {}
		
protected:

	void stop();

    void mount(Sensor* sensor);
    void unmount(Sensor* sensor);

    void recieve(QMap<QString, QString>* parts);


	uint renderingmode;

protected:

	XSocket server;

	QMap< QString, QMap< QString, Sensor* > > sensors;
	
	QQueue< QString > commands;
	QMutex outputMutex;
	
	bool isConnected;
	bool keepRunning;

	QString model, name, position, rotation;
    QUuid uniqueID;
		
};

#endif
