#ifndef BEHAVIOR_H
#define BEHAVIOR_H

#include <QMap>
#include <QString>

#include "Motion.h"
#include "Manifold.h"

class Sensor;

class Behavior;


enum BehaviorType {
	WAITFORLASER,
    EXPLORE,
    GETVICTIMID,
    GETVICTIMSTATUS
};

class BehaviorController
{

public:

    BehaviorController(MotionController* motionController, Manifold* map);
	virtual ~BehaviorController();

    void notifyUpdate(Sensor* sensor);
    void changeTo(BehaviorType behavior);
	void changeToPrevious() {
		changeTo(previousBehaviorType);
	}

    void executeMotion(MotionType movement);
    
    Manifold* getMap() {
    		return map;
    }
    
    MotionController* getMotionController() {
    		return motionController;
    }


private:

    MotionController* motionController;
    Manifold* map;

    QMap< BehaviorType, Behavior* > behaviors;
    Behavior* currentBehavior;
	BehaviorType currentBehaviorType;
	BehaviorType previousBehaviorType;

};



class Behavior
{
public:

    Behavior(BehaviorController* controller, QString name, QString description) {
        this->controller  = controller;
        this->name        = name;
        this->description = description;
    }
    virtual ~Behavior() {}
    
    QString getName()        { return this->name; }
    QString getDescription() { return this->description; }

    virtual void processUpdate(Sensor* sensor) {}

protected:
    BehaviorController* controller;
    QString name;
    QString description;
};

class BehaviorWaitForLaser : public Behavior
{
public:

    BehaviorWaitForLaser(BehaviorController* controller) 
        : Behavior(controller, "WaitForLaser", "Stop moving until a laser observation comes in.") {
		this->controller->executeMotion(NONE);
	}
    virtual ~BehaviorWaitForLaser() {}

    virtual void processUpdate(Sensor* sensor);
};

class BehaviorExplore : public Behavior
{
public:

    BehaviorExplore(BehaviorController* controller) 
        : Behavior(controller, "Explore", "Random walk with obstacle avoidance") {}
    virtual ~BehaviorExplore() {}

    virtual void processUpdate(Sensor* sensor);
};



class BehaviorGetVictimID : public Behavior
{
public:
	int counter;

    BehaviorGetVictimID(BehaviorController* controller) 
        : Behavior(controller, "GetVictimID", "A victim was seen, but not yet identified") {
        	counter = 0;
    }
    virtual ~BehaviorGetVictimID() {}

    virtual void processUpdate(Sensor* sensor);
};



class BehaviorGetVictimStatus : public Behavior
{
public:
	int counter;

    BehaviorGetVictimStatus(BehaviorController* controller) 
        : Behavior(controller, "GetVictimStatus", "A victim was identified, but it's status was not yet retrieved") {
        counter = 0;
    }
    virtual ~BehaviorGetVictimStatus() {}

    virtual void processUpdate(Sensor* sensor);
};


#endif
