#ifndef EXCEPTION_H_
#define EXCEPTION_H_

#include <QByteArray>
#include <QString>

#include <iostream>

class Exception
{
	private:
		static std::ostream* output;
		
		QString error;
		Exception* cause;

		char* errorFile;
		int errorLine;
	
	public:
		Exception(const QString &error) {
			this->error = error;
			
			this->cause = NULL;

			errorFile = NULL;
			errorLine = -1;
		}

		Exception(const QString &error, char* file, int line) {
			this->error = error;
			
			this->cause = NULL;

			errorFile = file;
			errorLine = line;
		}
		
		Exception(const QString &error, char* file, int line, Exception &cause) {
			this->cause = &cause;
			this->error = error;

			errorFile = file;
			errorLine = line;
		}

		Exception(const QString &error, Exception &cause) {
			this->cause = &cause;
			this->error = error;
		}
		
		virtual ~Exception() {
			if(cause!=NULL)
				delete cause;
		}
		
		static void setOStream(std::ostream &out) {
			output = &out;
		}
		
		virtual QString report() {
			QString reportStr("Exception: ");
			reportStr.append(error);
			if(errorFile!=NULL)
			{
				reportStr.append(" at ");
				reportStr.append(errorFile);
				reportStr.append(":");
				reportStr.append(QString().setNum(errorLine));
			}
			return reportStr;
		}
		
		virtual void print() {
			std::ostream *out = NULL;
			if(output!=NULL)
				out = output;
			else
				out = &std::cout;
			
			QString rep = report();
			(*out) << rep.toAscii().data() << std::endl;
			
			if(cause!=NULL) {
				(*out)<<"Caused by"<<std::endl;
				cause->print();
			}
		}
};

#endif /*EXCEPTION_H_*/
