#ifndef MOTION_H
#define MOTION_H

#include <QMap>
#include <QString>

#include "PositionActuator.h"

#include "INUSensor.h"
#include "LaserRangeSensor.h"
#include "OdometrySensor.h"
#include "RFIDSensor.h"
#include "SonarSensor.h"
#include "TouchSensor.h"
#include "VictimRFIDSensor.h"

#include "ScanUtils.h"

#include <time.h>

enum MotionType {
    NONE, 
    JUSTGO,
    GOTOVICTIM
};



class Motion;

class MotionController
{

public:

    MotionController(PositionActuator* actuator);
	virtual ~MotionController();

    void notifyUpdate(Sensor* sensor);
    void execute(MotionType motion);

    void move(float translation, float rotation);
    
    MotionType current() {
    		return currentMotionType;
    }
    
    Point2 attractionPoint;

private:

    QMap< MotionType, Motion* > motions;
    MotionType currentMotionType;
    Motion*    currentMotion;

    PositionActuator* actuator;
};



class Motion
{
public:

    Motion(MotionController* controller) {
        this->controller = controller;
    }
    virtual ~Motion() {}

    virtual void activate() {}
    virtual void deactivate() {}
    virtual void processUpdate(Sensor* sensor);

protected:
    virtual void process(const INUData&        inu) {}
    virtual void process(const LaserData&      range) {}
    virtual void process(const OdometryData&   odometry) {}
    virtual void process(const QMap<QString, RFIDData>&       rfid) {}
    virtual void process(const SonarData&      sonar) {}
    virtual void process(const TouchData&      touch) {}
    virtual void process(const QMap<QString, VictimRFIDData> & rfid) {}

    MotionController* controller;
};

// RELATIVE POINTS!
class MotionAttractToVictim : public Motion
{
public:
	bool avoiding;
	
	MotionAttractToVictim(MotionController* controller) : Motion(controller) {
		avoiding = false;
	}
	virtual ~MotionAttractToVictim() {}

protected:
	virtual void process(const QMap<QString, VictimRFIDData> & rfid);
	virtual void process(const LaserData& range);
};

class MotionExplore : public Motion
{
public:
    MotionExplore(MotionController* controller);
    virtual ~MotionExplore() {}

	virtual void deactivate();

protected:
    virtual void process(const LaserData& range);
    virtual void process(const INUData& inu);
    virtual void process(const SonarData& sonar);
private:

    void move();

    bool canGoNorth;
    bool eastLooksPromising;
    bool westLooksPromising;

    bool isTurningEast;
    bool isTurningWest;
    bool isRetreating;
    bool justRetreated;
    bool turningAfterRetreated;
    
    float freeSpace;
    
    clock_t startRetreat;
    clock_t startJustRetreated;

};





#endif
