#ifndef QUEUEDEVICE_H_
#define QUEUEDEVICE_H_

#include <QQueue>
#include <QMutex>
#include <QMutexLocker>
#include <QWaitCondition>

#include "Device.h"

/**
 * A simple extension of Device which contains a queue of received data types
 */

template< typename T >
class QueueDevice : public Device
{
	protected:
		
		QQueue< T > dataQueue;
		QMutex dataQueueMutex;
		QWaitCondition dataQueueCond;
		
		virtual void enqueueData(const T &data) {
			QMutexLocker lock(&dataQueueMutex);
			dataQueue.enqueue(data);
			dataQueueCond.wakeOne();
		}
	
	public:
		QueueDevice(QString name, QString type) : Device(name, type) {}
		
		virtual ~QueueDevice() {}
		
		T getData() {
			QMutexLocker lock(&dataQueueMutex);
			return dataQueue.dequeue();
		}
		
		T getNewestData() {
			QMutexLocker lock(&dataQueueMutex);
			return dataQueue.last();
		}
		
		void clearQueue() const {
			QMutexLocker lock(&dataQueueMutex);
			dataQueue.clear();
		}
		
		bool hasData() const {
			QMutexLocker lock(&dataQueueMutex);
			return !dataQueue.isEmpty();
		}
		
		// returns false if timed out, true otherwise
		bool waitForData(unsigned long timeout = ULONG_MAX) {
			QMutexLocker lock(&dataQueueMutex);
			return dataQueueCond.wait(&dataQueueMutex, timeout);
		}
};

#endif /*QUEUEDEVICE_H_*/
