#ifndef USARAGENT_H
#define USARAGENT_H

#include <QImage>
#include <QImageWriter>

#include "KeyboardSensor.h"

#include "Agent.h"
#include "Behavior.h"
#include "Motion.h"

#include "INUSensor.h"
#include "OdometrySensor.h"
#include "LaserRangeSensor.h"
#include "RFIDSensor.h"
#include "SonarSensor.h"
#include "TouchSensor.h"
#include "VictimRFIDSensor.h"

#include "HoughTransformer.h"

#include "ScanUtils.h"

#include "Manifold.h"
#include "ManifoldSlam.h"
#include "WeightedScanMatcher.h"

#include "ManifoldRenderer.h"
#include "QtManifoldRenderer.h"
#include "QtSimpleManifoldRenderer.h"

#include "UsarClient.h"

class UsarAgent : public Agent {

private:

public:

	UsarAgent();
	virtual ~UsarAgent();

    virtual void configure(QString model, QString name, QString position, QString rotation);

    virtual void saveMap(int pxPerMeter, QString filename);
    virtual void saveRFIDs(QString filename);
    virtual void saveVictims(QString filename);
    
    virtual void connectToMapServer(QString host, uint port);


protected:

    virtual void notifyUpdate(Sensor* sensor);

    virtual void observed(LaserData range);
    virtual void observed(QMap<QString, RFIDData> rfid);
    virtual void observed(QMap<QString, VictimRFIDData> rfid);


private:

	QMap< QString, RFIDData > previousRFIDs;
	QMap< QString, VictimRFIDData > previousVictimRFIDs;

    BehaviorController* behaviorController;
    MotionController*   motionController;

    PositionActuator* position;
    KeyboardSensor*   keyboard;

    INUSensor*        inu;
    OdometrySensor*   odometry;
    LaserRangeSensor* laser;
    RFIDSensor*       rfid;
    SonarSensor*      sonar;
    TouchSensor*      touch;
    VictimRFIDSensor* victimrfid;

    Manifold*            manifold;
    ManifoldSlam*        slam;
    WeightedScanMatcher* matcher;
    
    UsarClient*          usarclient;

    Pose2D               initialPose;
    
    HoughTransformer    detector;

};

#endif
