#include "LaserDevice3D.h"

#include <QDebug>

const QString LaserDevice3D::LASER3D("3DRangeScanner");

LaserDevice3D::LaserDevice3D(QString name)
	: QueueDevice<LaserData3D>(name, LASER3D)
{
}

LaserDevice3D::~LaserDevice3D()
{
}

void LaserDevice3D::process(QMap< QString, QString > *data) {

    LaserData3D newData;

    QStringList fovs   = (*data)["FOV"].split(",");
	QStringList ress   = (*data)["Resolution"].split(",");

    // the 3D range will come from bottom to top, from left to right
	QStringList ranges = (*data)["Range"].split(",");
    
    // the zeroth element seems to refer to the settings in vertical direction
    newData.vFov = fovs[0].toDouble();
    newData.hFov = fovs[1].toDouble();
	newData.vResolution = ress[0].toDouble();
	newData.hResolution = ress[1].toDouble();
	
    // add one to width and height to also include the zeroth scan
    newData.height = (int) (newData.vFov / newData.vResolution) + 1;
    newData.width  = (int) (newData.hFov / newData.hResolution) + 1;

	newData.data = new double*[newData.height];

    int count  = ranges.count();
    int length = newData.height * newData.width;

    if (count != length) {
        qDebug() << "Did not recieve the expected amount of data! Ignored. " << count << "!=" << length;
        qDebug() << "I thought I'd receive " << newData.height << " * " << newData.width << " ranges.";
        return;
    }

    int idx;
    for(int h = 0; h < newData.height; h++) {
		newData.data[h] = new double[newData.width];
	    for(int w = 0; w < newData.width; w++) {
            idx = h * newData.width + w;
		    newData.data[h][w] = ranges[idx].toDouble();
	    }
	}
	
	enqueueData(newData);
}

bool LaserDevice3D::acceptsMsg(QString type) {
	return type == "SEN";
}
