#include "LaserRangeSensor.h"
#include "Agent.h"

#include <iostream>
#include <QDebug>
#include <QStringList>


const QString LaserRangeSensor::LASERRANGE = "RangeScanner";

LaserRangeSensor::LaserRangeSensor(QString name)
: MultiStateSensor<LaserData>(LaserRangeSensor::LASERRANGE, name)
{
	needsConfiguration = true;

	// in meters
	minRange = 0.01;
	maxRange = 15;
}

LaserRangeSensor::~LaserRangeSensor()
{
}


void LaserRangeSensor::accept(QMap<QString, QString>* msg) {

	 if( isConfMessage(msg) ) {
		maxRange = getDouble(msg, "MaxRange");
		minRange = getDouble(msg, "MinRange");
		
	} else if( isSensorMessage(msg) ) {

		LaserData data;
		data.fov = (*msg)["FOV"].toDouble();
		data.resolution = (*msg)["Resolution"].toDouble();
		
		QStringList dists = (*msg)["Range"].split(",");
		data.length = dists.count();
		data.data = new double[data.length];
		
		for (int i = 0; i < data.length; i++) {
			double dist = dists[data.length-i-1].toDouble();
			data.data[i] = dist;
		}

		data.maxRange = maxRange;
		data.minRange = minRange;
		
		this->enqueueData(data);

		this->agent->notifyUpdate(this);
	}
}


