#include "OdometrySensor.h"
#include "Agent.h"

#include <iostream>
#include <QDebug>
#include <QStringList>


const QString OdometrySensor::ODOMETRY = "Odometry";



OdometrySensor::OdometrySensor(QString name)
: SingleStateSensor<OdometryData>(OdometrySensor::ODOMETRY, name)
{
    QMutexLocker lock(&this->mxdata);
    this->data.dx     = this->offset.dx     = 0.0;
    this->data.dy     = this->offset.dy     = 0.0;
    this->data.dtheta = this->offset.dtheta = 0.0;
}

OdometrySensor::~OdometrySensor() 
{
}


void OdometrySensor::accept(QMap<QString, QString>* msg) {
    bool changed = false;
    if (this->isSensorMessage(msg)) {
        QMutexLocker lock(&this->mxdata);
        QStringList list = (*msg)["Pose"].split(",");
        this->data.dx     = list[0].toDouble() - this->offset.dx;
        this->data.dy     = list[1].toDouble() - this->offset.dy;
        this->data.dtheta = list[2].toDouble() - this->offset.dtheta;
        changed = true;

		//qDebug() << "ODO  " << (*msg)["Pose"];
		//qDebug() << "ODO " << data.dx << "," << data.dx << "," << data.dtheta;

    } else if (this->isResponseMessage(msg)) {
        if (this->isStatusOK(msg)) {
            QMutexLocker lock(&this->mxdata);
            this->data.dx     = this->offset.dx     = 0.0;
            this->data.dy     = this->offset.dy     = 0.0;
            this->data.dtheta = this->offset.dtheta = 0.0;
            changed = true;
        }
    }
    if (changed)  {
        this->agent->notifyUpdate(this);
    }
}

void OdometrySensor::reset() {
    //this->sendSetCommand("Reset");
    QMutexLocker lock(&this->mxdata);
    this->offset      = this->data;
    this->data.dx     = 0.0;
    this->data.dy     = 0.0;
    this->data.dtheta = 0.0;

	//qDebug() << "RESET ODO";
}

