#include "StatusDevice.h"

#include <QDebug>
#include <QMutexLocker>
#include <QString>
#include <QStringList>

StatusDevice::StatusDevice()
	:Device(FALLBACK, ANY)
{
	data.time = 0;
	
	data.x = 0;
	data.y = 0;
	data.z = 0;
	
	data.rotX = 0;
	data.rotY = 0;
	data.rotZ = 0;
	
	data.velX = 0;
	data.velY = 0;
	data.velZ = 0;
	
	data.light = false;
	data.intensity = 0;
	data.battery = 0;
}

StatusDevice::~StatusDevice()
{
}

StatusData StatusDevice::getData() {
	QMutexLocker lock(&dataMutex);
	return data;
}

void StatusDevice::process(QMap<QString, QString> *data) {
//    qDebug() << "STATUS: ";
//	qDebug() << "- Time:           " << (*data)["Time"];
//	qDebug() << "- Location:       " << (*data)["Location"];
//	qDebug() << "- Orientation:    " << (*data)["Orientation"];
//	qDebug() << "- Velocity:       " << (*data)["Velocity"];
//	qDebug() << "- LightToggle:    " << (*data)["LightToggle"];
//	qDebug() << "- LightIntensity: " << (*data)["LightIntensity"];
//	qDebug() << "- Battery:        " << (*data)["Battery"];
	
	double time = (*data)["Time"].toDouble();
	
	double x,y,z;
	{
		QStringList list = (*data)["Location"].split(",");
		x = list[0].toDouble();
		y = list[1].toDouble();
		z = list[2].toDouble();
	}
	
	double rotx,roty,rotz;
	{
		QStringList list = (*data)["Orientation"].split(",");
		rotx = list[0].toDouble();
		roty = list[1].toDouble();
		rotz = list[2].toDouble();
	}
	
	double velx,vely,velz;
	{
		QStringList list = (*data)["Velocity"].split(",");
		velx = list[0].toDouble();
		vely = list[1].toDouble();
		velz = list[2].toDouble();
	}
	
	bool light = (*data)["LightToggle"] != "False";
	
	int intensity = (*data)["LightIntensity"].toInt();
	
	int battery = (*data)["Battery"].toInt();
	
	// now lock the mutex and set new data
	QMutexLocker lock(&dataMutex);
	this->data.time = time;
	
	this->data.x = x;
	this->data.y = y;
	this->data.z = z;
	
	this->data.rotX = rotx;
	this->data.rotY = roty;
	this->data.rotZ = rotz;
	
	this->data.velX = velx;
	this->data.velY = vely;
	this->data.velZ = velz;
	
	this->data.light = light;
	this->data.intensity = intensity;
	this->data.battery = battery;
}

bool StatusDevice::acceptsMsg(QString type) {
	return type == "STA";
}

