#include "VictimRFIDDevice.h"

#include <iostream>
#include <QDebug>
#include <QStringList>

#include "VictimRFIDData.h"

const QString VictimRFIDDevice::VICTIMRFID("VictRFID");
const QString VictimRFIDDevice::FALSEALARM("FalsePositive");
const QString VictimRFIDDevice::NOID("None");
const QString VictimRFIDDevice::NOSTATUS("None");

VictimRFIDDevice::VictimRFIDDevice()
	:Device(ANY,VICTIMRFID)
{
}

VictimRFIDDevice::~VictimRFIDDevice()
{
}


void VictimRFIDDevice::process(QMap<QString, QString> *data) {
	VictimRFIDData rfid;

    rfid.ID = (*data)["ID"];
    rfid.status = (*data)["Status"];

    /* decision schema:
     * - falsealarm      -> we see a false alarm
     * - nostatus + noid -> we see something, maybe false alarm when we get closer
     * - nostatus + id   -> we see a victim 
     * - status + id     -> we see a victim + status
     * 
     * Note: to see the status but no id is impossible.
     */

    if (rfid.ID == VictimRFIDDevice::FALSEALARM) {
        rfid.level = CanSeeFalseAlarm;
    } else if (rfid.status == VictimRFIDDevice::NOSTATUS) {
        if (rfid.ID == VictimRFIDDevice::NOID) {
            rfid.level = CanSee;
        } else {
            rfid.level = CanSeeVictim;
        }
    } else {
        rfid.level = CanSeeVictimStatus;
    }

    QStringList list = (*data)["Location"].split(",");
    rfid.dx = list[0].toDouble();
    rfid.dy = list[2].toDouble();
    rfid.dz = list[2].toDouble();

    qDebug() << "I SAW A VICTIM!";

}

bool VictimRFIDDevice::acceptsMsg(QString type) {
	return type == "SEN";
}

