/* START LOCATIONS:

Orange:	    0.4,2.1,1.4

TestRoom:   -10,-14,1.4 (behind victim)
	        5,18,1.4    (default)
Black:   	1.2,-2.5,1.8

*/


#include <QApplication>
#include <QtDebug>
#include <QFile>

#include <iostream>

#include "tclap/CmdLine.h"

#include "Agent.h"
#include "UsarAgent.h"


using namespace TCLAP;
using namespace std;

int _port;
bool _sleek;
QString _host;
QString _model;
QString _name;
QString _position;
QString _rotation;
QString _logfile;
QString _inlogfile;

QString _mapserver;
int _mapserverport;

void parseOptions(int argc, char** argv);

int main(int argc, char* argv[]) {

	parseOptions(argc,argv);

    Random::seed();

	QApplication app(argc, argv, true);
	QFile log;
	QFile inlog;

    Agent* agent = new UsarAgent();
    agent->configure(_model, _name, _position, _rotation);
	if (_sleek) {
		agent->setRenderingMode(Agent::RENDERINGMODE_SLEEK);
	} else {
		agent->setRenderingMode(Agent::RENDERINGMODE_SIMPLE);
	}
    
    if( _logfile.length() > 0 && _inlogfile.length() > 0 )
    {
    		qDebug() << "ERROR: can either read or write logs at a time";
    		return 1;
    }
    
    if( _logfile.length() > 0 ) {
    		log.setFileName(_logfile);
    		if( log.open(QIODevice::WriteOnly | QFile::Truncate) ) {
    			qDebug() << "logging to file: " << _logfile;
    			agent->logTo(&log);	
    		}
    }
    
    if( _inlogfile.length() > 0 ) {
    		inlog.setFileName(_inlogfile);
    		if( inlog.open(QIODevice::ReadOnly) ) {
    			qDebug() << "reading logfile: " << _inlogfile;
    			agent->logFrom(&inlog);
    		}
    }
    
	try {
		if( _mapserver.length() > 0 )
	    {
	    		((UsarAgent*)agent)->connectToMapServer(_mapserver,_mapserverport);
	    }
		
		agent->connect(_host, _port);
		agent->start();
		agent->wait();
		
		// 100 pixels per meter
        agent->saveRFIDs("rfids.log");
        agent->saveVictims("victims.log");
        agent->saveMap(100, "map.png");

    } catch(Exception e) {
		qDebug() << "Error:" << e.report();
		return 1;
	}

    return 1;
}





void parseOptions(int argc, char** argv)
{
	try {
		CmdLine cmd("Command line options for rescuecontrol.", ' ', "0.1" );

	    // Define arguments
	    ValueArg<string> logfile("o", "log", "output logfile", false, "", "string");
	    cmd.add(logfile);
	    ValueArg<string> inputlogfile("i", "inlog", "input logfile", false, "", "string");
	    cmd.add(inputlogfile);
		SwitchArg sleek("x", "sleek", "specify to use sleek drawing", false);
	    cmd.add(sleek);

	    ValueArg<string> rotation("r", "spawn_rotation", "the spawn rotation of the robot (Default: in x direction)", false, "0.0,0.0,0.0", "string");
	    cmd.add(rotation);
	    ValueArg<string> position("s", "spawn_position", "the spawn position of the robot (Default: start in orange arena)", false, "0.4,2.1,1.8", "string");
	    cmd.add(position);
	    ValueArg<string> name("n", "name", "a friendly name for the robot, (Default: Jimmy)", false, "Jimmy", "string");
	    cmd.add(name);
	    ValueArg<string> model("m", "model", "the type of robot to spawn, (Default: P2AT)", false, "USARBot.P2AT", "string");
	    cmd.add(model);
	    ValueArg<string> mapserver("g", "mapserver", "the name of the mapserver", false, "", "string");
	    cmd.add(mapserver);
	    ValueArg<int> mapserverport("b", "mapserverport",  "the port to connect to at the mapserver (Default: 5000)", false, 5000, "int");
	    cmd.add(mapserverport);
	    ValueArg<int> port("p", "port",  "the port to connect to (Default: 3000)", false, 3000, "int");
	    cmd.add(port);
	    UnlabeledValueArg<string> host("host", "the host to connect to (Default: localhost)", "localhost", "string");
	    cmd.add(host);

	    // Parse the command line.
	    cmd.parse(argc, argv);

	    // Retrieve variables
	    _port = port.getValue();
        _host = _host.fromStdString(host.getValue());
        _name = _name.fromStdString(name.getValue());
        _model = _model.fromStdString(model.getValue());
        if (!_model.startsWith("USARBot.")) {
            _model.prepend("USARBot.");
        }
        _position = _position.fromStdString(position.getValue());
        _rotation = _rotation.fromStdString(rotation.getValue());
        _logfile  = _logfile.fromStdString(logfile.getValue());
        _inlogfile  = _inlogfile.fromStdString(inputlogfile.getValue());
		_sleek    = sleek.getValue();

		_mapserver = _mapserver.fromStdString(mapserver.getValue());
		_mapserverport = mapserverport.getValue();
	} 
    catch (ArgException& e)
	{
        QString err = err.fromStdString("ERROR: " + e.error() + " " + e.argId());
        qDebug() << err;
    }
}
