
#include "CubeVisitor.h"



CubeVisitor::CubeVisitor() {}

CubeVisitor::~CubeVisitor() {}



void CubeVisitor::visitRoot(
	const OctreeCell* pRootCell,
	const OctreeData& octreeData
)
{
	if (pRootCell != 0)	{
		pRootCell->visit(octreeData, *this);
	}
}


void CubeVisitor::visitBranch(
	const OctreeCell* subCells[8],
	const OctreeData& octreeData
)
{
	// step through subcells (can be in any order)
	// subcell numbering:
	//    y z       6 7
	//    |/   2 3  4 5
	//     -x  0 1
	//
	// (in binary:)
	//    y z           110 111
	//    |/   010 011  100 101
	//     -x  000 001
	//
	for (dword i = 0;  i < 8;  ++i)	{
		const OctreeCell* pSubCell = subCells[i];

        // avoid null subcells
		if (pSubCell != 0) {
			// make subcell OctreeData: using octreeData argument and subcell index
			const OctreeData subCellData(octreeData, i);
			// continue visit traversal (can choose not to)
			pSubCell->visit(subCellData, *this);
		}

    }
}


void CubeVisitor::visitLeaf (
	const Array<const Cube*>& items,
	const OctreeData&         octreeData
)
{
	// make some short aliases
	const Vector3f& lower(octreeData.getBound().getLowerCorner());
	const Vector3f& upper(octreeData.getBound().getUpperCorner());

	// write leaf bound
	std::cout << "leaf: level(" << octreeData.getLevel() << ") ";
	std::cout << "lower(" << lower.getX() << ", " <<
	                         lower.getY() << ", " <<
	                         lower.getZ() << ") ";
	std::cout << "upper(" << upper.getX() << ", " <<
	                         upper.getY() << ", " <<
	                         upper.getZ() << ")\n";

	// write items
	for (dword i = 0, end = items.getLength();  i < end;  ++i) {
		// make some short aliases
		const Vector3f& lower(items[i]->getPosition());
		const Vector3f  upper(items[i]->getPosition() + items[i]->getDimensions());

		// write item
		std::cout << "   item: ";
		std::cout << "lower(" << lower.getX() <<
		                 ", " << lower.getY() <<
		                 ", " << lower.getZ() << ") ";
		std::cout << "upper(" << upper.getX() <<
		                 ", " << upper.getY() <<
		                 ", " << upper.getZ() << ")\n";
	}

	std::cout << "\n";
}
