#include <qpoint.h>
#include "controlbox.h"

ControlBox::ControlBox(QString title, QWidget * parent=0)
	:QWidget(parent)
{
	mvTitle=title;
	mvLayout = new QVBoxLayout();
	this->setLayout(mvLayout);
	mvLayout->setSpacing(0);
	mvLayout->setMargin(0);

	mvHead = new ControlBoxHead(NULL,title);
	mvLayout->addWidget(mvHead);
	
	mvContentBox = new QWidget();
	mvContentLayout = new QVBoxLayout();
	mvContentBox->setLayout(mvContentLayout);
	mvContentLayout->setSpacing(0);
	mvContentLayout->setMargin(0);
	
	mvLayout->addWidget(mvContentBox);

	mvLayout->setStretchFactor(mvHead,0);

	connect(mvHead,SIGNAL(toggled(bool)), this, SLOT(toggled(bool)));
	
}

ControlBox::~ControlBox()
{
	qDebug("ControlBox deconstructor called");
}

void ControlBox::addWidget(QString inName, QWidget * inWidget)
{
	mvContentLayout->addWidget(inWidget);
	mvChildDict.insert(inName, inWidget);
}

void ControlBox::collapse()
{
	mvHead->toggle(false);
	mvContentBox->hide();
}

void ControlBox::hideControl(QString inName)
{
	QWidget* temp = NULL;
	temp= mvChildDict[inName];
	if(temp)
		temp->hide();
}

void ControlBox::expand()
{
	mvHead->toggle(true);
	mvContentBox->show();
	
	// also show all children
	QMapIterator<QString,QWidget*> it( mvChildDict );
	while( it.hasNext() ) {
		it.next();
		it.value()->show();
	}
}

void ControlBox::toggled(bool state)
{
	if(state)
		expand();
	else
		collapse();
}

void ControlBox::showControl(QString inName)
{
	QWidget* temp = NULL;
	temp= mvChildDict[inName];
	if(temp)
		temp->show();
}
