#include "RobotWrapper.h"

#include <QtDebug>

#include "TrailObservation.h"
#include "Util3D.h"

RobotWrapper::RobotWrapper(Robot * a, LaserDevice3D * d, StatusDevice * s, Map * m)
{
	agent = a;
	map = m;
	device = d;
	status = s;
}

RobotWrapper::~RobotWrapper()
{
}

void RobotWrapper::run() {
	if(agent == NULL || map == NULL) {
		qDebug() << "Can't start RobotWrapper, null robot or map";
		return;
	}
	
	Pose3D oldPose;
	oldPose.x=0;
	oldPose.y=0;
	oldPose.z=0;
	oldPose.rotX=0;
	oldPose.rotY=0;
	oldPose.rotZ=0;

	while(agent->isRunning()) {
		if(device->waitForData(1000)) { // 1sec
			LaserData3D data = device->getData();
			StatusData stdata = status->getData();
			
			Pose3D cur;
			cur.x = stdata.x;
			cur.y = stdata.y;
			cur.z = stdata.z;
			cur.rotX = stdata.rotX;
			cur.rotY = stdata.rotY;
			cur.rotZ = stdata.rotZ;
			
			Pose3D diffPose = diff(cur, oldPose);
			
			TrailObservation * obs = new TrailObservation(data, diffPose);
			
			map->addObservation(name,obs);
			
			oldPose = cur;
		}
	}
}

Pose3D RobotWrapper::diff(Pose3D& p1, Pose3D& p2) {
	// the pose of p1 as seen from the position p2
	Matrix4 p2m;
	p2m.invFromPose3D(p2);
	
	Vector3 p1v(p1.x, p1.y, p1.z);
	Vector3 v = p2m.multiply(p1v);
	
	Pose3D ans;
	ans.x = v.x;
	ans.y = v.y;
	ans.z = v.z;
	
	ans.rotX = p1.rotX - p2.rotX;
	while(ans.rotX >= 2*M_PI)
		ans.rotX -= 2*M_PI;
	while(ans.rotX < 0)
		ans.rotX += 2*M_PI;
		
	ans.rotY = p1.rotY - p2.rotY;
	while(ans.rotY >= 2*M_PI)
		ans.rotY -= 2*M_PI;
	while(ans.rotY < 0)
		ans.rotY += 2*M_PI;
		
	ans.rotZ = p1.rotZ - p2.rotZ;
	while(ans.rotZ >= 2*M_PI)
		ans.rotZ -= 2*M_PI;
	while(ans.rotZ < 0)
		ans.rotZ += 2*M_PI;
		
	return ans;
}
