#ifndef OCTREE_H_
#define OCTREE_H_

#include <QList>
#include <QString>

#include "Util3D.h"

#ifdef USE_OPENGL
#include <QtOpenGL>
#endif

struct Ray {
	Vector3 origin;
	Vector3 direction;
	float length;
	float noise;
	char ordering;
};

class Octree
{
private:

	Point3 center;
	
	float edgeLength;

	bool isHit;
	
	static float minEdgeLength;
	static const float THRESHOLD_IS_FREE;
	static const float FLOAT_INF;
	
	// we have 8 children
	Octree* children[8];
	
	// probability of being occupied
	float occupancy;
	float hasVictim;
	float hasRFID;
	
	// parent node
	Octree* parent;
	
	// helper function to
	void recursiveDrawScanLine(Ray&, Vector3&, Vector3&);
	
	char firstNode(Vector3&, Vector3&);
	char nextNode(char, Vector3&);
	
	void insertChild(char);
	Vector3& combine(char , Vector3& , Vector3& , Vector3& );
	
	// internal constructor for child node
	Octree(float , Point3 , Octree* );
	
#ifdef USE_OPENGL
	// GL display list
	GLuint displayList;
#endif

public:
	Octree(float);
	Octree(float, float);
	Octree(float, float, Point3);
	virtual ~Octree();
	
	void setParent(Octree*);
	
	bool isFree();
	bool isLeaf();
	
	Octree* getChild(int);
	
	void insertVictim(Point3&);
	
	// used on the root
	void drawScanLine(Ray);
	
	
#ifdef USE_OPENGL
	void drawGL();
#endif
};

#endif /*OCTREE_H_*/
