#include <QUuid>

#include "Graph.h"


Graph::Graph() {
}
Graph::~Graph() {
	QMapIterator<QUuid, GraphLink*> linkIterator(this->links);
    while (linkIterator.hasNext()) {
        linkIterator.next();
        delete linkIterator.value();
    }
    this->links.clear();

    QMapIterator<QUuid, GraphNode*> nodeIterator(this->nodes);
    while (nodeIterator.hasNext()) {
        nodeIterator.next();
        delete nodeIterator.value();
    }
    this->nodes.clear();
}
uint Graph::nodeCount() {
    return this->nodes.count();
}
uint Graph::linkCount() {
    return this->links.count();
}
QMapIterator<QUuid, GraphNode*> Graph::nodeIterator() {
    QMapIterator<QUuid, GraphNode*> iterator(this->nodes);
    return iterator;
}
QMapIterator<QUuid, GraphLink*> Graph::linkIterator() {
    QMapIterator<QUuid, GraphLink*> iterator(this->links);
    return iterator;
}
GraphNode* Graph::insertNode(GraphNode* node) {
    this->nodes.insert(node->getUniqueID(), node);
    return node;
}
GraphLink* Graph::insertLink(GraphLink* link) {
    this->links.insert(link->getUniqueID(), link);
    return link;
}
GraphNode* Graph::getNode(const QUuid& uniqueID) {
    return this->nodes.value(uniqueID);
}
GraphLink* Graph::getLink(const QUuid& uniqueID) {
    return this->links.value(uniqueID);
}
GraphLink* Graph::getLink(GraphNode* fromNode, GraphNode* toNode) {
    if (this->hasNode(fromNode->getUniqueID()) && this->hasNode(toNode->getUniqueID())) 
        return fromNode->getLink(toNode);
    return 0;
}
GraphLink* Graph::getLink(const QUuid& fromNodeID, const QUuid& toNodeID) {
    if (this->hasNode(fromNodeID) && this->hasNode(toNodeID)) 
        return this->getNode(fromNodeID)->getLink(toNodeID);
    return 0;
}
bool Graph::hasNode(const QUuid& uniqueID) {
    return this->nodes.contains(uniqueID);
}
bool Graph::hasLink(const QUuid& uniqueID) {
    return this->links.contains(uniqueID);
}
bool Graph::hasLink(GraphNode* fromNode, GraphNode* toNode) {
    if (this->hasNode(fromNode->getUniqueID()) && this->hasNode(toNode->getUniqueID())) 
        return fromNode->hasLink(toNode);
    return false;
}
bool Graph::hasLink(const QUuid& fromNodeID, const QUuid& toNodeID) {
    if (this->hasNode(fromNodeID) && this->hasNode(toNodeID)) 
        return this->getNode(fromNodeID)->hasLink(toNodeID);
    return false;
}
void Graph::deleteNode(const QUuid& uniqueID) {
    GraphNode* node = this->nodes.value(uniqueID);
    this->nodes.remove(uniqueID);
    delete node;
}
void Graph::deleteLink(const QUuid& uniqueID) {
    GraphLink* link = this->links.value(uniqueID);
    this->links.remove(uniqueID);
    delete link;
}


QDataStream& operator<<(QDataStream& out, Graph* data)
{
	out << data->nodes.count();
	out << data->links.count();
	
	QMapIterator<QUuid, GraphNode*> nodes = data->nodeIterator();
	while(nodes.hasNext())
	{
		nodes.next();
		out << (nodes.value());
	}
	
	QMapIterator<QUuid, GraphLink*> links = data->linkIterator();
	while(links.hasNext())
	{
		links.next();
		out << (links.value());
	}
	
	return out;
}

QDataStream& operator>>(QDataStream& in, Graph* data)
{
	int numNodes, numLinks;
	data->nodes.clear();
	data->links.clear();
	
	in >> numNodes;
	in >> numLinks;
	
	for(int i=0; i<numNodes; i++)
	{
		// NO, this doesn't work, have to instanciate GraphNode subclass here?!?
	}
	
	return in;
}



GraphNode::GraphNode(Graph* graph, const QUuid& uniqueID) {
    this->graph    = graph;
    this->uniqueID = uniqueID;
}
GraphNode::~GraphNode() {
	QMapIterator<QUuid, GraphLink*> linkIterator(this->links);
    while (linkIterator.hasNext()) {
        linkIterator.next();
        delete linkIterator.value();
    }
    this->links.clear();
}
QUuid GraphNode::getUniqueID() {
    return this->uniqueID;
}
uint GraphNode::nodeCount() {
    return this->nodes.count();
}
uint GraphNode::linkCount() {
    return this->links.count();
}
QMapIterator<QUuid, GraphNode*> GraphNode::nodeIterator() {
    QMapIterator<QUuid, GraphNode*> iterator(this->nodes);
    return iterator;
}
QMapIterator<QUuid, GraphLink*> GraphNode::linkIterator() {
    QMapIterator<QUuid, GraphLink*> iterator(this->links);
    return iterator;
}
GraphLink* GraphNode::insertLink(const QUuid& uniqueID, GraphNode* toNode) {
    GraphLink* link = new GraphLink(this->graph, uniqueID, this, toNode);
    return this->graph->insertLink(link);
}
GraphLink* GraphNode::insertLink(const QUuid& uniqueID, const QUuid& toNodeID) {
    return this->insertLink(uniqueID, this->graph->getNode(toNodeID));
}
void GraphNode::deleteLink(GraphNode* toNode) {
    GraphLink* link = this->getLink(toNode);
    if (link)
        this->graph->deleteLink(link->getUniqueID());
}
void GraphNode::deleteLink(const QUuid& uniqueID) {
    if (this->hasLink(uniqueID))
        this->graph->deleteLink(uniqueID);
}
GraphLink* GraphNode::getLink(GraphNode* toNode) {
    QMapIterator<QUuid, GraphLink*> iterator = this->linkIterator();
    GraphLink* link; 
    GraphNode* node;
    while (iterator.hasNext()) {
        iterator.next();
        link = iterator.value();
        node = link->getOtherNode(this);
        if (node->getUniqueID() == toNode->getUniqueID())
            return link;
    }
    return 0;
}
GraphLink* GraphNode::getLink(const QUuid& uniqueID) {
    return this->links.value(uniqueID);
}
bool GraphNode::hasLink(GraphNode* toNode) {
    return this->nodes.contains(toNode->getUniqueID());
}
bool GraphNode::hasLink(const QUuid& uniqueID) {
    return this->links.contains(uniqueID);
}
void GraphNode::addLink(GraphLink* link) {
    this->links.insert(link->getUniqueID(), link);
    GraphNode* node = link->getOtherNode(this);
    this->nodes.insert(node->getUniqueID(), node);
}
void GraphNode::removeLink(GraphLink* link) {
    GraphNode* node = link->getOtherNode(this);
    this->nodes.remove(node->getUniqueID());
    this->links.remove(link->getUniqueID());
}


QDataStream& operator<<(QDataStream& out, const GraphNode*)
{
	return out;
}
QDataStream& operator>>(QDataStream& in, GraphNode*)
{
	return in;
}



GraphLink::GraphLink(Graph* graph, const QUuid& uniqueID, GraphNode* fromNode, GraphNode* toNode) {
    this->graph      = graph;
    this->uniqueID   = uniqueID;
    this->fromNode   = fromNode;
    this->fromNodeID = fromNode->getUniqueID();
    this->toNode     = toNode;
    this->toNodeID   = toNode->getUniqueID();

    this->fromNode->addLink(this);
    this->toNode->addLink(this);

}
GraphLink::~GraphLink() {
    this->fromNode->removeLink(this);
    this->toNode->removeLink(this);
}
QUuid GraphLink::getUniqueID() {
    return this->uniqueID;
}
GraphNode* GraphLink::getOtherNode(GraphNode* node) {
    QUuid nodeID = node->getUniqueID();
    if (nodeID == this->fromNodeID)
        return this->toNode;
    else if (nodeID == this->toNodeID)
        return this->fromNode;
    return 0;
}

QDataStream& operator<<(QDataStream& out, const GraphLink*)
{
	return out;
}
QDataStream& operator>>(QDataStream& in, GraphLink*)
{
	return in;
}

