#include "XServer.h"

#include <QtDebug>

XServer::XServer()
	: XSocket()
{
	int yes=1;
	if (setsockopt(sockfd,SOL_SOCKET,SO_REUSEADDR,(char*)&yes,sizeof(int)) == -1) {
		throw Exception("Couldn't set sockopt SO_REUSEADDR. " + lastError(), __FILE__, __LINE__);
	}
}

XServer::~XServer()
{
}

void XServer::listen(uint port, int allowedQueuedConnections)
{
	remoteAddr.sin_family = AF_INET;         // host byte order
    remoteAddr.sin_port = htons(port);     // short, network byte order
    remoteAddr.sin_addr.s_addr = INADDR_ANY; // automatically fill with my IP

    if ( ::bind(sockfd, (sockaddr *)&remoteAddr, sizeof(remoteAddr)) == -1) {
		aIsConnected = false;
        throw Exception("Couldn't bind to port. " + lastError(), __FILE__, __LINE__);
    }

    if ( ::listen(sockfd, allowedQueuedConnections) == -1) {
		aIsConnected = false;
        throw Exception("Couldn't start listening. " + lastError(), __FILE__, __LINE__);
    }

	aIsConnected = true;
}

// default XSocket implementation, overrideable
XSocket* XServer::newXSocket(int descriptor)
{
	return new XSocket(descriptor);
}

XSocket* XServer::accept(uint msec)
{
	fd_set sock_set;
	FD_ZERO(&sock_set);
	FD_SET( sockfd, &sock_set);
	timeval timeout;
	timeout.tv_sec = msec/1000;
	timeout.tv_usec = msec%1000 * 1000;
	
	int sel = select( sockfd+1,  &sock_set, NULL, NULL, &timeout );
	if( sel == -1 )
	{
		aIsConnected = false;
		throw Exception("Couldn't wait for new connection: " + lastError(), __FILE__, __LINE__ );
	}

	if( sel == 0 )
		return NULL; // nothing to accept
		
	int newSock = 0;
	sockaddr_in their_addr;
	int sin_size = sizeof(sockaddr_in);
	
	if ((newSock = ::WSAAccept(sockfd, (struct sockaddr *)&their_addr, &sin_size, NULL, NULL)) == -1) {
		aIsConnected = false;
		throw Exception("Couldn't accept new connection: " + lastError() , __FILE__, __LINE__);
	}
	qDebug() << "XServer: got connection from " << inet_ntoa(their_addr.sin_addr);
	
	return newXSocket(newSock);
	
}
