#include "Server.h"

#include <QtDebug>

ServerConnection::ServerConnection(XSocket* sock, Server* server)
	: Connection(sock)
{
	parent = server;
	
	// enter threaded event loop
	start();
}

ServerConnection::~ServerConnection()
{
}

void ServerConnection::run()
{
	Connection::run();

	qDebug() << "connection stopped";
	
	if(parent != NULL)
		parent->removeConnection(this);
}

Server::Server(uint port)
{	
	this->port = port;
	currentConnection = NULL;
	serverSocket.listen(port);
	
	qDebug() << "listening: " << port;

	start();
}

Server::~Server()
{
}

ServerConnection* Server::newServerConnection(XSocket* sock)
{
	return new ServerConnection(sock, this);
}

void Server::run()
{
	qDebug() << "start run";
	try {
		
		while( serverSocket.isConnected() )
		{
			for(int i=0; i<toBeDeleted.size(); i++)
				delete toBeDeleted[i];
			toBeDeleted.clear();
			
			XSocket* sock = serverSocket.accept(3000);
			if( sock != NULL )
			{
				qDebug() << "new connection";
				ServerConnection* conn = newServerConnection(sock);
				connections.append( conn );
				if( currentConnection == NULL )
					currentConnection = conn;
			}
			else
			{
				qDebug() << "no new connection";
			}
		}
	} catch(Exception e) {
		qDebug() << "Server::run: Exception: " << e.report();
	}
	qDebug() << "end run";
}

void Server::disconnect()
{
	serverSocket.disconnect();
	wait();
}

void Server::disconnectClients()
{
	QListIterator<ServerConnection*> conns(connections);
	while(conns.hasNext())
	{
		conns.next()->disconnect();
	}
}

void Server::removeConnection(ServerConnection* con)
{
	qDebug() << "removing connection from server";

	for(int i=0; i<connections.count(); i++)
		if( connections[i] == con )
		{
			qDebug() << "removing connection from list";
			connections.removeAt(i);
			toBeDeleted.append(con);
			return;
		}

	qDebug() << "didn't find this connection in the list!";
}

QListIterator<ServerConnection*> Server::connectionIterator()
{
	return QListIterator<ServerConnection*>(connections);
}

ServerConnection* Server::getConnection(int num)
{
	if( connections.size() >= num )
		return NULL;
		
	return connections[num];
}

int Server::countConnections()
{
	return connections.count();
}

void Server::broadcast(const QByteArray& data)
{
	QListIterator<ServerConnection*> it(connections);
	QUuid id;
	while(it.hasNext())
	{
		it.next()->send(id, data);
	}
}
