#include "UsarClient.h"

#include "DataOperators.h"

UsarClient::UsarClient(QString host, uint port)
	: Client(host, port)
{
}

UsarClient::~UsarClient()
{
}

void UsarClient::newPatchAdded(ScanObservation* obs)
{
	try {
		submitPatch(obs, NULL, NULL);
	} catch(Exception e) {
		qDebug() << "newPatchAdded(obs): " << e.report();
	}
}

void UsarClient::newPatchAdded(ScanObservation* obs, QUuid otherId, Matrix3 cov)
{
	try {
		submitPatch(obs, &otherId, &cov);
	} catch(Exception e) {
		qDebug() << "newPatchAdded(obs, id, cov): " << e.report();
	}
}

void UsarClient::newVictimStatus(QString ID, QString status)
{
	Conversation conv;
	QByteArray msg;
	QDataStream out(&msg, QIODevice::WriteOnly);
	
	out << (quint8)1;
	
	out << ID;
	out << status;
	
	if( send(conv.getId(),msg) )
		qDebug() << "********* sent status update for " << ID << " to " << status << " successfully.";
}

void UsarClient::submitPatch(ScanObservation* obs, QUuid* otherPatch, Matrix3* cov)
{
	Conversation conv;
	QByteArray msg;
	QDataStream out(&msg, QIODevice::WriteOnly);
	
	out << (quint8)2;
	
	out << obs->getUniqueID();
	
	out << obs->rawpose.x;
	out << obs->rawpose.y;
	out << obs->rawpose.rot;
	
	out << obs->pose.x;
	out << obs->pose.y;
	out << obs->pose.rot;
	
	out << obs->maxdistance;
	out << obs->mindistance;
	
	out << obs->unit;
	out << obs->length;
	
	for( int i=0; i<obs->length; i++)
	{
		out << obs->range[i];
	}
	
	out << (obs->countRFIDs());
	{
		QMapIterator< QString, RFIDData > rfids = obs->rfidIterator();
		while( rfids.hasNext() )
		{
			rfids.next();
			out << rfids.value();
		}
	}
	
	out << (obs->countVictims());
	{
		QMapIterator< QString, VictimRFIDData > rfids = obs->victimIterator();
		while( rfids.hasNext() )
		{
			rfids.next();
			out << rfids.value();
		}
	}
	
	bool hasConnection = (otherPatch != NULL);
	
	
	out << hasConnection;
	
	if( hasConnection )
	{
		out << (*otherPatch);
		
		for(int i=0; i<3; i++)
		for(int j=0; j<3; j++)
			out << (*cov)(i,j);
			
		qDebug() << "******** added link along with patch";
	}
	
	
	if( send(conv.getId(),msg) )
		qDebug() << "********* sent patch " << obs->getUniqueID() << " successfully.";
}
