#ifndef MANIFOLDSLAM_H_
#define MANIFOLDSLAM_H_


#include <QUuid>
#include <QQueue>

#include "SlamStrategy.h"
#include "ManifoldRenderer.h"

#include "Manifold.h"

class ManifoldSlamListener
{
public:
	virtual ~ManifoldSlamListener() {}

	virtual void newPatchAdded(ScanObservation*) = 0;
	virtual void newPatchAdded(ScanObservation*, QUuid, Matrix3) = 0;
	
	virtual void newVictimStatus(QString, QString) = 0;
};

class ManifoldSlam : public SlamStrategy
{

private:
    
	// these determine when to extend the manifold
    static const float EXT_PATCHPOSEDISTANCE;
    static const float EXT_PATCHPOSEROTATION;
    static const float EXT_TRANSLATIONCOVARIANCE;
    static const float EXT_ROTATIONCOVARIANCE;
    static const uint  EXT_NUMCORRESPONDENCIES;

	// these determine which patches shouldn't added to prevent manifold damage
    static const float MAX_TRANSLATIONCOVARIANCE;
    static const float MAX_ROTATIONCOVARIANCE;
    static const uint  MAX_NUMCORRESPONDENCIES;

	QList<ManifoldSlamListener*> listeners;

public:

	ManifoldSlam(ScanMatcher* matcher, Manifold* manifold);
	virtual ~ManifoldSlam();

	bool processScan(ScanObservation* scan);
    void processVictim(VictimRFIDData victim);

	void extendWithPatch(ScanObservation* newPatch);
	void extendWithLink(QUuid, QUuid, Matrix3);
	
	void addListener(ManifoldSlamListener*);


	//void render(ManifoldRenderer* renderer);

    ScanObservation* getCurrentPatch();
    Pose2D           getCurrentPose();

	bool mergeIslands(QUuid source, QUuid target);

private:

    void extendManifold(ScanObservation* newScan);
    void extendManifold(ScanObservation* newScan, ScanObservation* atScan, const Matrix3& mcov);

    void moveToPatch(ScanObservation* patch);
    QList<ScanObservation*>* getSubmap(ScanObservation* patch);
	bool localize();

    void closeLoop(ScanObservation* source, ScanObservation* target);
    
    void refitPatches(ManifoldNode* node, QList<QUuid>& done);
    void refitPatches(ManifoldNode* sourceNode, ManifoldNode* targetNode);


    // Manifold representation
    Manifold* manifold;

    // current state vars
    QList<ScanObservation*>* currentSubmap;
    ScanObservation*         currentPatch;
    Pose2D                   currentPose;


    uint   number, iMax, nMax, nSum;
    double sMax;

};

#endif 
