#include "Util3D.h"

#ifdef USE_OPENGL
#include <QtOpenGl>
#endif

#include <QtDebug>

QDataStream& operator<<(QDataStream& out, const Pose3D& pose)
{
	out << pose.x;
	out << pose.y;
	out << pose.z;
	out << pose.rotX;
	out << pose.rotY;
	out << pose.rotZ;
	
	return out;
}
QDataStream& operator>>(QDataStream& in, Pose3D& pose)
{
	in >> pose.x;
	in >> pose.y;
	in >> pose.z;
	in >> pose.rotX;
	in >> pose.rotY;
	in >> pose.rotZ;
	
	return in;
}

Matrix4 Matrix4::operator*(Matrix4& right) {
	Matrix4 ret;
	float sum = 0;
	
	for(int i=0; i<4; i++)
		for(int j=0; j<4; j++) {
			sum = 0;
			for(int k=0; k<4; k++)
				sum += nums[index(i,k,4)] * right.nums[index(k,j,4)];
			
			nums[index(i,j,4)] = sum;
		}
	
	return ret;
}

void Matrix4::loadIdentity() {
	for(int i=0; i<4; i++)
		for(int j=0; j<4; j++)
			if(i!=j)
				nums[index(i,j,4)] = 0;
			else
				nums[index(i,j,4)] = 1;
}

// STILL BROKEN, DONT USE!
//Matrix4 Matrix4::inv() {
//	Matrix4 ret;
//	
//	for(int i=0; i<3; i++)
//		for(int j=0; j<3; j++)
//			ret(i,j) = minorDet2(i,j);
//	
//	float det3 = nums[index(0,0,4)] * nums[index(1,1,4)] * nums[index(2,2,4)]
//			- nums[index(0,0,4)] * nums[index(1,2,4)] * nums[index(2,1,4)]
//			- nums[index(0,1,4)] * nums[index(1,0,4)] * nums[index(2,2,4)]
//			+ nums[index(0,1,4)] * nums[index(1,2,4)] * nums[index(2,0,4)]
//			+ nums[index(0,2,4)] * nums[index(1,0,4)] * nums[index(2,1,4)]
//			- nums[index(0,2,4)] * nums[index(1,1,4)] * nums[index(2,0,4)];
//	
//	qDebug() << "mat det: " <<det3;
//	
//	if(det3 == 0) {
//		qDebug() << "Matrix is singular, can't compute inverse!";
//		return Matrix4(); // singular matrices don't have an inverse
//	}
//	
//	qDebug() << "cofactors:";
//	for(int i=0; i<4; i++)
//		qDebug() << ret(i,0) << ret(i,1) << ret(i,2) << ret(i,3);
//	
//	
//	for(int i=0; i<3; i++)
//		for(int j=0; j<3; j++)
//			ret(i,j) = ret(i,j) / det3;
//	
//			
//	// now for the translation part
//	
//	for(int i=0; i<3; i++) {
//		ret(i,3) = - ( ret(i,0)*(*this)(0,3) + ret(i,1)*(*this)(1,3) + ret(i,2)*(*this)(2,3) );
//	}
//	
//	return ret;
//}

// determinant of the 2x2 minor Matrix ij
//float Matrix4::minorDet2(int i, int j) {
//	float a[4];
//	
//	for(int x=0; x<4; x++)
//		a[x]=0.0f;
//	
//	for(int x=0, xi=0; x<2 && xi<3; x++, xi++) {
//		if(xi==i) xi++;
//		for(int y=0, yj=0; y<2 && yj<3; y++, yj++) {
//			  if(yj==j) yj++;
//			  a[index(x,y,2)] = nums[index(xi,yj,4)];
//		}
//	}
//	
//	qDebug() << "minor "<<i<<j;
//	qDebug() << a[0] << a[1];
//	qDebug() << a[2] << a[3];
//	
//	
//	float det = a[index(0,0,2)] * a[index(1,1,2)] - a[index(0,1,2)] * a[index(1,0,2)];
//	
//	qDebug() << "det: " << det;
//	return det;
//}

void Matrix4::rotate(float rx, float ry, float rz) {
	Matrix4 rot;
	rot.setRotation(rx,ry,rz);
	
	(*this) = rot.multiply((*this));
}

void Matrix4::translate(float x, float y, float z) {
	Matrix4 t;
	t.setTranslation(x,y,z);
	
	(*this) = t.multiply((*this));
}

void Matrix4::invRotate(float rx, float ry, float rz) {
	Matrix4 rot;
	rot.setInvRotation(rx,ry,rz);
	
	(*this) = this->multiply(rot);
}

void Matrix4::invTranslate(float x, float y, float z) {
	Matrix4 t;
	t.setTranslation(-x,-y,-z);
	
	(*this) = this->multiply(t);
}

void Matrix4::setRotation(float rx, float ry, float rz) {
	float cx = cos(rx);
	float sx = sin(rx);
	float cy = cos(ry);
	float sy = sin(ry);
	float cz = cos(rz);
	float sz = sin(rz);
	
	(*this)(0,0) = cy * cz;
	(*this)(0,1) = -sz * cy;
	(*this)(0,2) = sy;
	(*this)(1,0) = cz * sy * sx + sz * cx;
	(*this)(1,1) = cz * cx - sz * sy * sx;
	(*this)(1,2) = -cy * sx;
	(*this)(2,0) = sz * sx - cz * sy * cx;
	(*this)(2,1) = sz * sy * cx + sx * cz;
	(*this)(2,2) = cy * cx;
}

void Matrix4::setInvRotation(float rx, float ry, float rz) {
	float cx = cos(rx);
	float sx = sin(rx);
	float cy = cos(ry);
	float sy = sin(ry);
	float cz = cos(rz);
	float sz = sin(rz);
	
	// actually transpose of rotation matrix
	
	(*this)(0,0) = cy * cz;
	(*this)(0,1) = cz * sy * sx + sz * cx;
	(*this)(0,2) = -cz * sy * cx + sz * sx;
	(*this)(1,0) = -sz * cy;
	(*this)(1,1) = -sz * sy * sx + cz * cx;
	(*this)(1,2) = sz * sy * cx + cz * sx;
	(*this)(2,0) = sy;
	(*this)(2,1) = -cy * sx;
	(*this)(2,2) = cy * cx;
}

void Matrix4::setTranslation(float x, float y, float z) {
	(*this)(0,3) = x;
	(*this)(1,3) = y;
	(*this)(2,3) = z;
}

Vector3 Matrix4::multiply(Vector3 &v) {
	Vector3 ans;
	ans.x = v.x*(*this)(0,0) + v.y*(*this)(0,1) + v.z*(*this)(0,2) + (*this)(0,3);
	ans.y = v.x*(*this)(1,0) + v.y*(*this)(1,1) + v.z*(*this)(1,2) + (*this)(1,3);
	ans.z = v.x*(*this)(2,0) + v.y*(*this)(2,1) + v.z*(*this)(2,2) + (*this)(2,3);
	
	return ans;
}

Matrix4 Matrix4::multiply(Matrix4 &m) {
	Matrix4 ans;
	
	double sum=0;
	for(int i=0; i<4; i++) {
		for(int j=0; j<4; j++) {
			sum=0;
			for(int k=0; k<4; k++)
				sum += (*this)(i,k) * m(k,j);
			ans(i,j) = sum;
		}
	}
	
	return ans;
}

Matrix4 & Matrix4::operator=(const Matrix4& m) {
	//qDebug() << "Matrix4 assgn!";
	for(int i=0; i<16; i++)
		nums[i] = m.nums[i];
	
	return *this;
}

// *****************************************************************************************
// * FROM the book "Computer Graphics using Open GL", by F.S. Hill, Jr., Pearson Education *
// *****************************************************************************************

#ifdef USE_OPENGL
Camera::Camera()
{
	viewAngle=aspect=nearDist=farDist=0.0;
	set(Point3(), Point3(1,0,0), Vector3(0,0,1));
}

void Camera :: setModelViewMatrix(void)
{ // load modelview matrix with existing camera values

	//qDebug("new camera eye: x:"+QString::number(eye.x)+" y:"+QString::number(eye.y)+" z:"+QString::number(eye.z));
	float m[16];
	Vector3 eVec(eye.x, eye.y, eye.z); // a vector version of eye 
	m[0] =  u.x; m[4] =  u.y; m[8]  =  u.z;  m[12] = -eVec.dot(u);
	m[1] =  v.x; m[5] =  v.y; m[9]  =  v.z;  m[13] = -eVec.dot(v);
	m[2] =  n.x; m[6] =  n.y; m[10] =  n.z;  m[14] = -eVec.dot(n);
	m[3] =  0;   m[7] =  0;   m[11] =  0;    m[15] = 1.0;
	glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(m); // load OpenGLs modelview matrix
}

void Camera:: set(Point3 Eye, Point3 look, Vector3 up)
{
	// create a modelview matrix and send it to OpenGL
	eye.set(Eye); // store the given eye position
	n.set(eye.x - look.x, eye.y - look.y, eye.z - look.z); // make n
	u=(up.cross(n)); // make u = up X n
	n.normalize(); u.normalize(); // make them unit length
	v=(n.cross(u));  // make v =  n X u
	setModelViewMatrix(); // tell OpenGL
	
	// u = y
	// n = -x
	// v = z
}

void Camera :: roll(float angle)
{
	// u = y
	// n = -x
	// v = z
	
	// roll the camera through angle degrees
	float cs = cos(3.14159265/180 * angle);
	float sn = sin(3.14159265/180 * angle);
	
	Vector3 t(u); // remember old u
	u = cs*t + sn*v;
	v = -sn*t + cs*v;
	
	setModelViewMatrix();
}

void Camera :: pitch(float angle)
{
	// u = y
	// n = -x
	// v = z
	
	// roll the camera through angle degrees
	float cs = cos(3.14159265/180 * angle);
	float sn = sin(3.14159265/180 * angle);
	
	Vector3 t(n); // remember old u
	n = cs*t + sn*v;
	v = -sn*t + cs*v;
	
	setModelViewMatrix();
}

void Camera :: yaw(float angle)
{
	// u = y
	// n = -x
	// v = z
	
	// roll the camera through angle degrees
	float cs = cos(3.14159265/180 * angle);
	float sn = sin(3.14159265/180 * angle);
	
	Vector3 t(u); // remember old u
	u = cs*t + sn*n;
	n = -sn*t + cs*n;
	
	setModelViewMatrix();
}

void Camera::rotate(float angle)
{
	// u = y
	// n = -x
	// v = z
	
	// rotate only around z
	rotateZ(u,angle);
	rotateZ(n,angle);
	rotateZ(v,angle);
	
	setModelViewMatrix();
}

void Camera::rotateZ(Vector3& vec, float angle)
{
	float cs = cos(3.14159265/180 * angle);
	float sn = sin(3.14159265/180 * angle);
	
	float x = vec.x;
	float y = vec.y;
	
	vec.x = cs*x - sn*y;
	vec.y = cs*y + sn*x;
}


// slide( forward, right, up )
void Camera::slide(float delN, float delU, float delV) // slide it
{
	// u = y
	// n = -x
	// v = z
	
	eye.x += delU*u.x + delV*v.x - delN*n.x;
	eye.y += delU*u.y + delV*v.y - delN*n.y;
	eye.z += delU*u.z + delV*v.z - delN*n.z;
	
	setModelViewMatrix();
}

void Camera::setShape(float vAng, float asp, float nearD, float farD)
{
	viewAngle=vAng;
	aspect=asp;
	nearDist=nearD;
	farDist=farD;
	glMatrixMode(GL_PROJECTION);
	gluPerspective(vAng,asp,nearD,farD);
}

void Camera::setEye(float x, float y, float z)
{
	eye.set(x,y,z);
	setModelViewMatrix();
}

void Camera::move(float x, float y, float z)
{
	eye.x+=x;
	eye.y+=y;
	eye.z+=z;
	setModelViewMatrix();
}
#endif

Vector3 operator*(const float & left, const Vector3 & right)
{
	Vector3 temp(left*right.x, left*right.y, left*right.z);
	return temp;
}

Vector3 Vector3::operator-(const Vector3 & right)
{
	Vector3 temp(x-right.x,y-right.y,z-right.z);
	return temp;
}

Vector3 Vector3::operator+(const Vector3 & right)
{
	Vector3 temp(x+right.x,y+right.y,z+right.z);
	return temp;
}

Vector3& Vector3::operator=(const Vector3 & right)
{
	x=right.x;
	y=right.y;
	z=right.z;
	return *this;
}
