#include <QDebug>
#include <QFile>
#include <QString>
#include <QStringList>

#include "time.h"

#include "LogScanPlayer.h"
#include "Exception.h"
#include "ScanUtils.h"


#include "ManifoldSlam.h"


LogScanPlayer::LogScanPlayer(const QString logfile, SlamStrategy* slam, ManifoldRenderer* map)
: ScanPlayer(slam, map)
{
    this->_logfile = logfile;
    this->previousScan = 0;
    this->iMax = 0;

    this->source = 0;
    this->target = 0;
}

LogScanPlayer::~LogScanPlayer()
{
}


void LogScanPlayer::play() {

    try {

        QFile log(this->_logfile);
        if (!log.open(QIODevice::ReadOnly | QIODevice::Text))
            throw Exception("Could not open file! " + _logfile);


        QTextStream in(&log);
        uint current = 0;

        this->iMax = 0;
        this->nSum = 0;

        time_t start = clock();

        while (!in.atEnd()) {
            current ++;
            qDebug() << "Processing scan " << current << " ...";
            processLine(current, in.readLine());
        }

        time_t stop = clock();

        double elapsed = (double) (stop - start) / CLOCKS_PER_SEC;
        double average = elapsed / (double) current;

        qDebug() << "=============================================================";
        qDebug() << "Timing:";
        qDebug() << "- " << current << " scans took " << elapsed << " seconds";
        qDebug() << "- " << average << " seconds per scan on average";
        qDebug() << "- " << (1/average) << " scans per second on average";
        //qDebug() << "Iterations:";
        //qDebug() << "- " << current << " scans took " << this->nSum << " iterations in total";
        //qDebug() << "- " << (this->nSum / current) << " iterations per scan on average";
        //qDebug() << "Scan " << this->iMax << " was hardest to match:";
        //qDebug() << "- " << this->nMax << " iterations took " << this->sMax << " seconds";
        qDebug() << "Done.";

    } catch(Exception e) {
        qDebug() << "Error:" << e.report();
    }

}


void LogScanPlayer::processLine(const uint number, const QString line) {

    QStringList parts = line.split(" ");

	LaserData range;
    double distance;
	//range.fov = (*data)["FOV"].toDouble();
	//range.resolution = (*data)["Resolution"].toDouble();
	range.length = parts.count() - 3;
	range.data = new double[range.length];
	
	for (int i = 0; i < range.length; i++) {
        distance      = parts[i+3].toDouble();
		range.data[i] = distance;
	}

    this->currentScan = new ScanObservation(1000, parts[0].toFloat(), parts[1].toFloat(), parts[2].toFloat(), range);
    this->slam->processScan(this->currentScan);

    //if (number == 1)
    //    this->target = this->currentScan;
    //if (number == 641)
    //    this->source = this->currentScan;

    //if (source && target) {
    //    ((ManifoldSlam*) this->slam)->closeLoop(source, target);
    //    source = 0;
    //    target = 0;
    //}



    /*

    bool skipslam = false;

    if (skipslam || !previousScan) {
        // first round, initialize current pose
        this->currentPose.x   = currentScan->pose.x;
        this->currentPose.y   = currentScan->pose.y;
        this->currentPose.rot = currentScan->pose.rot;
    }

    if (!skipslam && previousScan) {

        // match this scan against the previous one
        time_t       start  = clock();
        MatchResult* result = this->matcher->matchScans(previousScan, currentScan);
        time_t       stop   = clock();

        double elapsed = (double) (stop - start) / CLOCKS_PER_SEC;

        this->nSum += result->numiterations;
        if (result->numiterations > this->nMax || this->iMax == 0) {
            this->iMax = number;
            this->nMax = result->numiterations;
            this->sMax = elapsed;
        }

        //qDebug() << "-- " << result->numiterations << " iterations took " << elapsed << " seconds";
        qDebug() << "-- logfile dpose    = " << result->rawdpose.x << result->rawdpose.y << result->rawdpose.rot;
        qDebug() << "-- weighted dpose   = " << result->smdpose.x << result->smdpose.y << result->smdpose.rot << "(" << result->numiterations << " iterations)";

        // the returned dpose is wrt the previousScan
        // so here it is converted to map (global) coords
        Matrix3 rotmx;
        Vector2 vdpose(result->smdpose.x, result->smdpose.y);
        rotmx.setRotation(this->currentPose.rot);
        vdpose = rotmx.multiply(vdpose);

        // update current (global) pose
        this->currentPose.x   = this->currentPose.x   + vdpose.x;
        this->currentPose.y   = this->currentPose.y   + vdpose.y;
        this->currentPose.rot = this->currentPose.rot + result->smdpose.rot;

        delete result;
        result = 0;
    }

    this->map->renderScan(this->currentPose, currentScan, number);

    if (previousScan)
        delete previousScan;

    this->previousScan = this->currentScan;

    //*/

}

