#include <QtGui>

#include "MapWindow.h"


const double MapWindow::DEFAULTWIDTH  = 1400;
const double MapWindow::DEFAULTHEIGHT = 800;
const double MapWindow::DEFAULTZOOM   = .8;


MapWindow::MapWindow()
{
    setAttribute(Qt::WA_DeleteOnClose);
    isUntitled = true;
    zoomScale = 1.0;

    this->mapImageLabel = new QLabel;
    this->mapImageLabel->setBackgroundRole(QPalette::Base);
    this->mapImageLabel->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
    this->mapImageLabel->setScaledContents(true);

    this->resize(DEFAULTWIDTH * DEFAULTZOOM, DEFAULTHEIGHT * DEFAULTZOOM);
    this->setBackgroundRole(QPalette::Dark);
    this->setWidget(this->mapImageLabel);

}

MapWindow::~MapWindow() {
}


void MapWindow::setScanMatcher(ScanMatcher* matcher) {
}

void MapWindow::setScanPlayer(ScanPlayer* player) {
}

bool MapWindow::loadImageFile(const QString &fileName)
{
    if (!fileName.isEmpty()) {
        this->mapImage.load(fileName);
        if (this->mapImage.isNull()) {
            QMessageBox::information(this, tr("MapWindow"), tr("Cannot load %1.").arg(fileName));
            return false;
        }
    }

    this->mapImageLabel->setPixmap(QPixmap::fromImage(mapImage));
    this->mapImageLabel->adjustSize();

    this->update();

    return true;
}

bool MapWindow::loadLogFile(const QString &fileName)
{
    this->mapImage = QImage(DEFAULTWIDTH, DEFAULTHEIGHT, QImage::Format_ARGB32_Premultiplied);

    ManifoldRenderer*  renderer = new QtManifoldRenderer(&this->mapImage);
    ScanMatcher*  matcher  = (ScanMatcher*)  new WeightedScanMatcher();
    Manifold*     manifold = new Manifold();
    //SimpleSlam* slam       = new SimpleSlam(matcher);
    ManifoldSlam* slam     = new ManifoldSlam(matcher, manifold);
    ScanPlayer*   player   = (ScanPlayer*)   new LogScanPlayer(fileName, slam, renderer);

    player->play();

	slam->render(renderer);

    this->mapImageLabel->setPixmap(QPixmap::fromImage(mapImage));
    this->mapImageLabel->adjustSize();

    this->zoomTo(DEFAULTZOOM);

    delete renderer;
    delete matcher;
    delete slam;
    delete manifold;
    delete player;

    return true;
}


bool MapWindow::save()
{
    if (isUntitled) {
        return saveAs();
    } else {
        return saveFile(curFile);
    }
}

bool MapWindow::saveAs()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save As"),
                                                    curFile);
    if (fileName.isEmpty())
        return false;

    return saveFile(fileName);
}

bool MapWindow::saveFile(const QString &fileName)
{
    //QFile file(fileName);
    //if (!file.open(QFile::WriteOnly | QFile::Text)) {
    //    QMessageBox::warning(this, tr("MDI"),
    //                         tr("Cannot write file %1:\n%2.")
    //                         .arg(fileName)
    //                         .arg(file.errorString()));
    //    return false;
    //}

    //QTextStream out(&file);
    //QApplication::setOverrideCursor(Qt::WaitCursor);
    //out << toPlainText();
    //QApplication::restoreOverrideCursor();

    //setCurrentFile(fileName);
    return true;
}

QString MapWindow::userFriendlyCurrentFile()
{
    return strippedName(curFile);
}


void MapWindow::closeEvent(QCloseEvent *event)
{
    if (maybeSave()) {
        event->accept();
    } else {
        event->ignore();
    }
}

void MapWindow::documentWasModified()
{
//    setWindowModified(document()->isModified());
}

bool MapWindow::maybeSave()
{
    //if (document()->isModified()) {
    //    int ret = QMessageBox::warning(this, tr("MDI"),
    //                 tr("'%1' has been modified.\n"
    //                    "Do you want to save your changes?")
    //                 .arg(userFriendlyCurrentFile()),
    //                 QMessageBox::Yes | QMessageBox::Default,
    //                 QMessageBox::No,
    //                 QMessageBox::Cancel | QMessageBox::Escape);
    //    if (ret == QMessageBox::Yes)
    //        return save();
    //    else if (ret == QMessageBox::Cancel)
    //        return false;
    //}
    return true;
}

void MapWindow::setCurrentFile(const QString &fileName)
{
    curFile = QFileInfo(fileName).canonicalFilePath();
    isUntitled = false;
    //document()->setModified(false);
    setWindowModified(false);
    setWindowTitle(userFriendlyCurrentFile() + "[*]");
}

QString MapWindow::strippedName(const QString &fullFileName)
{
    return QFileInfo(fullFileName).fileName();
}




void MapWindow::zoomBy(double factor) {
    this->zoomTo(this->zoomScale * factor);
}

void MapWindow::zoomTo(double scale) {

    Q_ASSERT(this->mapImageLabel->pixmap());

    this->mapImageLabel->resize(scale * this->mapImageLabel->pixmap()->size());

    this->adjustScrollBar(this->horizontalScrollBar(), scale);
    this->adjustScrollBar(this->verticalScrollBar(), scale);

    this->zoomScale = scale;
}

void MapWindow::adjustScrollBar(QScrollBar *scrollBar, double scale) {
    scrollBar->setValue(int(scale * scrollBar->value() + ((scale - 1) * scrollBar->pageStep()/2)));
}


void MapWindow::paintEvent(QPaintEvent* event) {
}
