#include "time.h"

#include "SimpleSlam.h"


SimpleSlam::SimpleSlam(ScanMatcher* matcher, ManifoldRenderer* renderer)
: SlamStrategy(matcher)
{
    this->renderer = renderer;
    this->previousScan = 0;
    this->number = 0;
}

SimpleSlam::~SimpleSlam()
{
    if (this->currentScan) {
        delete currentScan;
        currentScan = 0;
    }
}

bool SimpleSlam::processScan(ScanObservation* scan) 
{
    this->number++;

    this->currentScan = scan;

    if (!previousScan) {
        // first round, initialize current pose
        this->currentPose.x   = currentScan->pose.x;
        this->currentPose.y   = currentScan->pose.y;
        this->currentPose.rot = currentScan->pose.rot;

    } else {
        // match this scan against the previous one
        time_t       start  = clock();
        MatchResult* result = this->matcher->matchScans(previousScan, currentScan);
        time_t       stop   = clock();

        double elapsed = (double) (stop - start) / CLOCKS_PER_SEC;

        this->nSum += result->numIterations;
        if (result->numIterations > this->nMax || this->iMax == 0) {
            this->iMax = number;
            this->nMax = result->numIterations;
            this->sMax = elapsed;
        }

        //qDebug() << "-- " << result->numiterations << " iterations took " << elapsed << " seconds";
        qDebug() << "-- logfile dpose    = " << result->rawdpose.x << result->rawdpose.y << result->rawdpose.rot;
        qDebug() << "-- weighted dpose   = " << result->smdpose.x << result->smdpose.y << result->smdpose.rot << "(" << result->numIterations << " iterations)";

        // the returned dpose is wrt the previousScan
        // so here it is converted to map (global) coords
        Matrix3 rotmx;
        Vector2 vdpose(result->smdpose.x, result->smdpose.y);
        rotmx.setRotation(this->currentPose.rot);
        vdpose = rotmx.multiply(vdpose);

        // update current (global) pose
        this->currentPose.x   = this->currentPose.x   + vdpose.x;
        this->currentPose.y   = this->currentPose.y   + vdpose.y;
        this->currentPose.rot = this->currentPose.rot + result->smdpose.rot;

        delete result;
        result = 0;

    }

    this->renderer->renderScan(this->currentPose, currentScan, number);

    if (previousScan)
        delete previousScan;

    this->previousScan = this->currentScan;

    return false;
}

